;	======================================
;	       Signaldiorama mit Tiny12
;	======================================

;	-------------------------------
;	Port
;	-------------------------------
;	B0: Output address bit 0
;	B1: Output address bit 1
;	B2: Output address bit 2
;	B3: Output address bit 3
;	B4: Output Data bit

;	-------------------------------
;	Timer
;	-------------------------------
;	Timer0: Interrupt every 0,0017s, Prescaler 1

;	---------------------------------
;	Standard definitions for ATtiny12
;	---------------------------------
	.include "tn12def.inc"

;	-------------------------------
;	Constant definitions
;	-------------------------------
	.equ	wc1max	=250		;end value wait counter 1
	.equ	wc2max	=95		;end value wait counter 2
	.equ	maxdata	=14		;Anzahl aller Signalkombinationen
	.equ	port	=portb
	.equ	pin	=pinb
	.equ	ddr	=ddrb
	.equ	odata	=16		;data input of 74159

;	-------------------------------
;	Register definitions
;	-------------------------------
	.def	stor	=r0		;storage variable for LPM
	.def	flags	=r1		;flag storage
	.def	temp	=r16		;temporary storage variable
	.def	ttemp1	=r17		;temporary timer variable 1
	.def	ttemp2	=r18		;temporary timer variable 2
	.def	data1	=r19		;data variable 1
	.def	data2	=r20		;data variable 2
	.def	bitloop	=r21		;bit counter
	.def	waico	=r22		;wait counter variable
	.def	waico2	=r23		;wait counter variable 2
	.def	proco	=r24		;program counter

;	---------------------------------
;	Program start, Interrupt vectors
;	---------------------------------
	.cseg
	.org	0x0000

reset:	rjmp	initia			;program start
exint0:	reti
pncnge:	reti
t0ovfl:	rjmp	timer0			;time counter
eerdy:	reti
ancomp:	reti

;Initialize I/O and interrupts
initia:	cli				;disable interrupts
	ldi	temp,0x3e
	out	osccal,temp		;calibrate oscillator
	ldi	temp,0x1f
	out	ddr,temp		;set outputs
	clr	temp
	out	port,temp		;clear outputs
	ldi	temp,1<<toie0
	out	timsk,temp		;Timer0 interrupt
	ldi	temp,1<<cs00
	out	tccr0,temp		;Timer0 overflow, prescaler = 1
	sei				;enable interrupts

;Lamp test
	ser	data1
	ser	data2
	rcall	wait

;Main program
main:	clr	proco			;counter for Signal data
	ldi	zl,low(datas*2)
	ldi	zh,high(datas*2)	;Pointer Z on Signal data
main1:	lpm
	mov	data1,stor		;get data byte 1
	ldi	temp,1
	add 	zl,temp
	ldi	temp,0
	adc	zh,temp			;Pointer Z increment
	lpm
	mov	data2,stor		;get data byte 2
	rcall	wait
	ldi	temp,1
	add 	zl,temp
	ldi	temp,0
	adc	zh,temp
	inc	proco
	cpi	proco,maxdata		;all Signal data read?
	brlo	main1
	rjmp	main

wait:	clr	waico2			;waiting loop ca. 5 sec.
wait1:	clr	waico
wait2:	cpi	waico,wc1max
	brlo	wait2
	inc	waico2
	cpi	waico2,wc2max
	brlo	wait1
	ret

timer0: ;Timer0 ISR, use flags,bitloop,ttemp1 and ttemp2 only here
	in	flags,sreg	;save flags
	inc	waico
	inc	bitloop
	cpi	bitloop,8
	brlo	ti1
	cpi	bitloop,16
	brlo	ti4
	clr	bitloop
	mov	ttemp1,data1	;copy data to work registers
	mov	ttemp2,data2

ti1:	sbrs	ttemp1,7	;first data byte
	sbr	bitloop,odata
	rol	ttemp1
	rjmp	tend

ti4:	sbrs	ttemp2,7	;second data byte
	sbr	bitloop,odata
	rol	ttemp2

tend:	out	port,bitloop	;output bit data
	cbr	bitloop,odata
	out	sreg,flags	;restore flags
	reti

;Signal data      rrygwryg   yyggyygg
datas:	.db	0b11000100,0b00001100	;Hp00,Hp0,Vr0
	.db	0b11000011,0b11000110	;Hp00,Hp2+Vr0,Vr2
	.db	0b00110011,0b01100110	;Hp2,Hp2+Vr2,Vr2
	.db	0b00110100,0b00001100	;Hp2,Hp0,Vr0
	.db	0b11000100,0b00001100	;Hp00,Hp0,Vr0
	.db	0b10001100,0b00001100	;Hp0+Sh1,Hp0,Vr0
	.db	0b11000100,0b00001100	;Hp00,Hp0,Vr0
	.db	0b11000011,0b11000110	;Hp00,Hp2+Vr0,Vr2
	.db	0b11000100,0b00001100	;Hp00,Hp0,Vr0
	.db	0b00010100,0b00001100	;Hp1,Hp0,Vr0
	.db	0b11000100,0b00001100	;Hp00,Hp0,Vr0
	.db	0b11000001,0b11000011	;Hp00,Hp1+Vr0,Vr1
	.db	0b00010001,0b00110011	;Hp1,Hp1+Vr1,Vr1
	.db	0b00010100,0b00001100	;Hp1,Hp0,Vr0